#!/bin/sh
#
# FiwixOS _VERSION_ installation script.
# Copyright 2018-2022, Jordi Sanfeliu. All rights reserved.
# Distributed under the terms of the Fiwix License.
#

OSVERSION="_VERSION_"

KVERSION=$(uname -r)
MAXPARTSIZE=4097
umask 0022

RED="echo -en \\033[1;31m"
GREEN="echo -en \\033[1;32m"
YELLOW="echo -en \\033[1;33m"
CYAN="echo -en \\033[1;36m"
BLUE="echo -en \\033[1;34m"
BGBLUE="echo -en \\033[1;44m"
DEFAULT="echo -en \\033[m"	# \\033[0;39m
REVERSE="echo -en \\033[7m"
BRIGHT="echo -en \\033[1;39m"

DIALOG="dialog --ascii-lines"
HEADER="FiwixOS $OSVERSION Installation"
LINES=24
CONSOLE="/dev/tty12"
RESPONSE="/tmp/dialog.txt"
keymap=
BOOTDEV=$(df / | grep dev | awk '{print $1}')
HDLIST=$(grep " hd.$" /proc/partitions | awk '{print $4}')
HDNUM=$(echo $HDLIST | wc -w)

# number of partitions and their sizes (in MiB)
nparts=2
bootsize=8
rootsize=max
homesize=max


scroll() {
	local l
	for l in $(seq 1 1 $LINES) ; do
		echo
	done
}

press_enter() {
	local in
	$REVERSE "Press <ENTER> to $1"
	read in
	$DEFAULT
}

cleanup() {
	cd /
	rm -f $RESPONSE
	umount /mnt/cdrom 2>/dev/null
	umount /mnt/ram 2>/dev/null
	umount /mnt/disk 2>/dev/null
}

abort() {
	$DEFAULT
	echo
	echo "Installation aborted."
	cleanup
	exit 1
}

system() {
	# setsid sh -c 'exec command <> /dev/tty2 >&0 2>&1'

	local result
	$@ <> $CONSOLE >&0 2>&1
	result=$?
	return $result
}

get_disksize() {
	local disk=$1
	local hd=$(echo $disk | sed 's#/dev/##')
	local kb=$(grep "$hd$" /proc/partitions | awk '{print $3}')
	local size=$(($kb / 1024))
	echo $size
}

welcome() {
	echo "				  W E L C O M E"
	echo
	echo "			to the installation of FiwixOS $OSVERSION"
	echo "			    for the i386 architecture"
	echo
	echo "		    Copyright (c) 2018-2022 by Jordi Sanfeliu"
	echo "			      <jordi@fibranet.cat>"
	echo
	echo
	echo "				  kernel v$KVERSION"
	echo
	echo
	echo
	echo
	echo
	echo
	echo
	press_enter "continue"
}

info1() {
	local text="
FiwixOS is a Fiwix distribution, an operating system made from
a software collection  that is based upon the Fiwix kernel. It
basically comprises the Fiwix kernel, GNU tools, libraries and
additional software.

Fiwix is an operating system kernel, written by Jordi Sanfeliu
from scratch, based on the UNIX architecture and fully focused
on being POSIX compatible. It is designed and developed mainly
as a hobby OS but also for educational purposes.

It runs on most 32-bit  ISA PCs in existence.  This means that
it should  run even on an old 80386 with 2MiB of RAM. Although
the minimum memory recommended is 128MiB.

"
	$DIALOG --backtitle "$HEADER" --title "" --msgbox "$text" 0 0
}

info2() {
	local text="
Please keep in mind  that this  kernel is not yet  suited  for
production  use and  may  well  have serious  bugs  and broken
features, which have not yet been identified or resolved.


*****************************
*** USE AT YOUR OWN RISK! ***
*****************************

"
	$DIALOG --backtitle "$HEADER" --title "[ WARNING ]" --msgbox "$text" 0 0
}

info3() {
	local text="The installation  program  will  start  shortly.  You  will be
prompted for the information needed to install FiwixOS in your
hard disk drive. Default values are provided where useful.

You can interrupt the  installation  procedure at  any time by
pressing  <CTRL-C>,  then  type  'install.sh'  and  start  the
installation again.

Please be  patient and  read the  instructions  on  the screen
carefully.

Thanks.
"
	$DIALOG --backtitle "$HEADER" --title "" --msgbox "$text" 0 0
}

error() {
	echo
	echo
	echo "An error occurred during the installation."
	echo "Switch to /dev/tty12 to get more information."
	echo
	press_enter "exit"
	return 1
}

check_disk_drive() {
	if [ -z "$HDLIST" ] ; then
		local text="Sorry,  no hard disk drives have been detected in your system.


Please, make sure you have an IDE/ATA compatible hard disk and
restart the installation procedure.

Thanks.


"
		$DIALOG --backtitle "$HEADER" --title "[ ERROR ]" --infobox "$text" 0 0
		press_enter "exit"
		exit 1
	fi
}

load_keymap() {
	case $1 in
		1) keymap=cf.bmap ;;
		2) keymap=dk.bmap ;;
		3) keymap=nl.bmap ;;
		4) keymap=fi.bmap ;;
		5) keymap=fr.bmap ;;
		6) keymap=de.bmap ;;
		7) keymap=gr.bmap ;;
		8) keymap=hu.bmap ;;
		9) keymap=it.bmap ;;
		10) keymap=no.bmap ;;
		11) keymap=pl.bmap ;;
		12) keymap=pt.bmap ;;
		13) keymap=es.bmap ;;
		14) keymap=uk.bmap ;;
		15) keymap=us.bmap ;;
		*) return 1 ;;
	esac

	loadkmap </etc/i18n/$keymap
	return 0
}

keyboard_selection() {
	local text="
Please specify the keyboard layout from the list below."
	local kbd
	local result

	> $RESPONSE
	$DIALOG --backtitle "$HEADER" --title "[ KEYBOARD SELECTION ]" --menu "$text" \
		0 40 10 \
		1 "Canadian-French" \
		2 "Danish" \
		3 "Dutch" \
		4 "Finish" \
		5 "French" \
		6 "German" \
		7 "Greece" \
		8 "Hungarian" \
		9 "Italian" \
		10 "Norwegian" \
		11 "Polish" \
		12 "Portuguese" \
		13 "Spanish-Catalan" \
		14 "UK-English" \
		15 "US-English" 2>$RESPONSE

	result=$?
	case $result in
		0) kbd=$(cat $RESPONSE) ;;
		*) abort ;;
	esac
	load_keymap $kbd
	pass=2
}

target_media_selection() {
	local menulist
	local result
	local text="
You need to specify in which hard disk you want to
install the FiwixOS $OSVERSION. A minimal of 750MiB
of capacity is required.

The following is the list of all hard disk drives
detected in your system:
"
	for disk in $HDLIST ; do
		local size=$(get_disksize "/dev/$disk")
		if [ "$size" -lt 750 ] ; then
			size="(${size}MiB - INSUFFICIENT CAPACITY!)"
			
		else
			size="(${size}MiB)"
		fi
		menulist="$menulist /dev/$disk $size"
	done

	> $RESPONSE
	$DIALOG --backtitle "$HEADER" --title "[ TARGET MEDIA SELECTION ]" --menu "$text" 0 0 1 $menulist 2>$RESPONSE

	result=$?
	case $result in
		0) TARGET=$(cat $RESPONSE) ;;
		1) pass=$(expr $pass - 1) ; return $pass ;;
	esac

	disksize=$(get_disksize $TARGET)
	pass=3
}

last_warning() {
	local text="
If you continue all files currently on the hard disk
'$TARGET' will be erased!

Do you really want to continue ('yes' or 'no')?
"
	local wish
	local result

	while : ; do
		> $RESPONSE
		$DIALOG --backtitle "$HEADER" --title "[ LAST WARNING ]" --inputbox "$text" 0 0 2>$RESPONSE
		result=$?
		case $result in
			0) wish=$(cat $RESPONSE) ;;
			1) pass=$(expr $pass - 1) ; return $pass ;;
		esac

		case $wish in
			yes) pass=5 ; break ;;
			no) pass=$(expr $pass - 1) ; break ;;
		esac
	done

	disksize=$(($disksize - ($bootsize + 1)))
	if [ "$disksize" -gt $MAXPARTSIZE ] ; then
		nparts=3
		rootsize=$MAXPARTSIZE
		disksize=$(($disksize - $rootsize))
	fi
	if [ "$disksize" -gt $MAXPARTSIZE ] ; then
		homesize=$MAXPARTSIZE
		disksize=$(($disksize - $homesize))
	fi
}

# RAMdisk is used to store temporary files in /tmp
setup_ramdisk() {
	system "$SOURCE/sbin/mke2fs -q -r 0 -m 0 -b 1024 /dev/ram0" || error || exit 1
	mount -t ext2 /dev/ram0 /mnt/ram || error || exit 1
	mkdir -p /mnt/ram/tmp
}

initializing_disk_partition_block() {
	system "dd if=/dev/zero of=$TARGET bs=1c count=512" || error || exit 1
	echo w | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
}

create_boot_partition() {
	(echo n ; echo p ; echo 1 ; echo 1 ; echo +${bootsize}M ; echo w) | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
}

create_root_partition() {
	if [ "$rootsize" = "max" ] ; then
		(echo n ; echo p ; echo 2 ; echo ; echo ; echo w) | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
	else
		(echo n ; echo p ; echo 2 ; echo ; echo +${rootsize}M ; echo w) | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
	fi
}

create_home_partition() {
	if [ "$nparts" -gt 2 ] ; then
		if [ "$homesize" = "max" ] ; then
			(echo n ; echo p ; echo 3 ; echo ; echo ; echo w) | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
		else
			(echo n ; echo p ; echo 3 ; echo ; echo +${homesize}M ; echo w) | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
		fi
	fi
}

make_boot_fs() {
	(echo t ; echo 1 ; echo 81 ; echo w) | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
	system "$SOURCE/sbin/mkfs.minix -n 30 ${TARGET}1" || error || exit 1
}

make_root_fs() {
	(echo t ; echo 2 ; echo 81 ; echo w) | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
	system "$SOURCE/sbin/mke2fs -q -r 0 -b 1024 ${TARGET}2" || error || exit 1
}

make_home_fs() {
	if [ "$nparts" -gt 2 ] ; then
		(echo t ; echo 3 ; echo 81 ; echo w) | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
		system "$SOURCE/sbin/mke2fs -q -r 0 -b 1024 ${TARGET}3" || error || exit 1
	fi
}

activate_boot() {
	(echo a ; echo 1 ; echo w) | /busybox fdisk $TARGET >$CONSOLE 2>$CONSOLE || error || exit 1
}

show_target_layout() {
	local text="
The following is the current partition layout in $TARGET:"

	fdisk -l $TARGET | $DIALOG --backtitle "$HEADER" --title "[ DEVICE LAYOUT ]" --programbox "$text" 18 77
}

start_copy() {
	local text="
All software packages in CD-ROM are going to be transferred on to your hard disk."

	$DIALOG --backtitle "$HEADER" --title "[ SOFTWARE INSTALLATION ]" --msgbox "$text" 0 0
}

mount_root_fs() {
	system "mount -t ext2 ${TARGET}2 /mnt/disk" || error || exit 1
	cd /mnt/disk
}

create_devices() {
	system "mkdir dev" || error || exit 1
	makedev mem c 1 1 640 mem
	makedev kmem c 1 2 640 kmem
	makedev null c 1 3 666 root
	makedev port c 1 4 640 kmem
	makedev zero c 1 5 666 root
	makedev full c 1 7 666 root
	makedev random c 1 8 644 root
	makedev urandom c 1 9 644 root
	makedev ram0 b 1 0 660 disk
	ln -s /dev/ram0 dev/ram

	makedev fd0 b 2 0 660 floppy
	makedev fd0h360 b 2 4 660 floppy
	makedev fd0h1200 b 2 8 660 floppy
	makedev fd0h720 b 2 12 660 floppy
	makedev fd0h1440 b 2 16 660 floppy
	makedev fd1 b 2 1 660 floppy
	makedev fd1h360 b 2 5 660 floppy
	makedev fd1h1200 b 2 9 660 floppy
	makedev fd1h720 b 2 13 660 floppy
	makedev fd1h1440 b 2 17 660 floppy

	makedev hda b 3 0 660 disk
	makedev hda1 b 3 1 660 disk
	makedev hda2 b 3 2 660 disk
	makedev hda3 b 3 3 660 disk
	makedev hda4 b 3 4 660 disk
	makedev hdb b 3 64 660 disk
	makedev hdb1 b 3 65 660 disk
	makedev hdb2 b 3 66 660 disk
	makedev hdb3 b 3 67 660 disk
	makedev hdb4 b 3 68 660 disk
	makedev hdc b 22 0 660 disk
	makedev hdc1 b 22 1 660 disk
	makedev hdc2 b 22 2 660 disk
	makedev hdc3 b 22 3 660 disk
	makedev hdc4 b 22 4 660 disk
	makedev hdd b 22 64 660 disk
	makedev hdd1 b 22 65 660 disk
	makedev hdd2 b 22 66 660 disk
	makedev hdd3 b 22 67 660 disk
	makedev hdd4 b 22 68 660 disk

	makedev console c 5 1 600 root
	makedev tty c 5 0 666 root
	makedev tty0 c 4 0 660 root
	makedev tty1 c 4 1 660 root
	makedev tty2 c 4 2 660 root
	makedev tty3 c 4 3 660 root
	makedev tty4 c 4 4 660 root
	makedev tty5 c 4 5 660 root
	makedev tty6 c 4 6 660 root
	makedev tty7 c 4 7 660 root
	makedev tty8 c 4 8 660 root
	makedev tty9 c 4 9 660 root
	makedev tty10 c 4 10 660 root
	makedev tty11 c 4 11 660 root
	makedev tty12 c 4 12 660 root
	makedev ttyS0 c 4 64 660 root
	makedev ttyS1 c 4 65 660 root
	makedev ttyS2 c 4 66 660 root
	makedev ttyS3 c 4 67 660 root

	makedev lp0 c 6 0 660 lp

	makedev fb0 c 29 0 660 root
	ln -s /dev/fb0 dev/fb

	mkfifo dev/fifo
	chmod 666 dev/fifo
}

create_system_dirs() {
	mkdir boot
	mkdir home
	mkdir mnt
	mkdir mnt/cdrom
	mkdir mnt/disk
	mkdir mnt/floppy
	mkdir proc
	mkdir root
	mkdir tmp
	mkdir -p usr/sbin
	ln -s usr/bin bin
	ln -s usr/sbin sbin
	mkdir -p var/log
	mkdir -p var/run
	mkdir -p var/spool
	chmod 1777 tmp

	touch var/log/wtmp
	touch var/log/btmp
}

makedev() {
	system "mknod /mnt/disk/dev/$1 $2 $3 $4" || error || exit 1
	chmod $5 /mnt/disk/dev/$1 || error || exit 1
	chgrp $6 /mnt/disk/dev/$1 || error || exit 1
}

install_system_software() {
	local text="
Please wait while the software is installed, this may take a while.
"

	# install the package manager first
	system "opkg install -V0 -o . --force-overwrite ${SOURCE}/install/pkgs/bin/opkg_0.6.0_i386.ipk" || error || exit 1

	for file in $(ls ${SOURCE}/install/base/* 2>/dev/null) \
		    $(ls ${SOURCE}/install/pkgs/bin/* 2>/dev/null) \
		    $(ls ${SOURCE}/install/extras/* 2>/dev/null) ; do

		# skip some packages
		case $file in
			*busybox*) continue ;;
			*kernel_${KVERSION}*) continue ;;
			*opkg_0.6.0*) continue ;;
		esac

		# check if it's a 'tar.bz2' or 'ipk'
		unset tar ipk
		tar=$(echo $file | sed 's/\.tar\.bz2$//')
		if [ "$tar" = "$file" ] ; then
			unset tar
			ipk=$(echo $file | sed 's/\(_i386\|_noarch\)\.ipk$//')
			if [ "$ipk" = "$file" ] ; then
				unset ipk
				echo "ERROR: unrecognized file '$file'."
			fi
		fi

		# is a tar.bz2
		if [ -n "$tar" ] ; then
			base=$(basename $file)
			base=$(echo $base | sed 's/\.tar\.bz2$//')
			echo "$base"
			system "tar jxf $file" || error || exit 1
		fi

		# is an ipk
		if [ -n "$ipk" ] ; then
			base=$(basename $file)
			base=$(echo $base | sed 's/\(_i386\|_noarch\)\.ipk$//')
			echo "$base"
			system "opkg install -V0 -o . --force-overwrite $file" || error || exit 1
		fi
	done | $DIALOG --backtitle "$HEADER" --title "[ INSTALLING SYSTEM SOFTWARE ]" --progressbox "$text" 19 50


	text="
Syncing buffers to disk ...

"
	$DIALOG --backtitle "$HEADER" --title "[ INSTALLING SYSTEM SOFTWARE ]" --infobox "$text" 0 0
	sync
}

configure_system_files() {
	# make sure symlink sh points to bash
	ln -sf bash usr/bin/sh

	usr/bin/chown -R root:root etc usr 2>/dev/null
	cp ${SOURCE}/install/README root
	chmod 700 root

	cat <<EOF >root/.bash_profile
# .bash_profile

# Get the aliases and functions
if [ -f ~/.bashrc ] ; then
	. ~/.bashrc
fi

# User specific environment and startup programs
PATH=$PATH:/usr/local/bin:/root/bin

export PATH

EOF

	cat <<EOF >root/.bashrc
# .bashrc

# Source global definitions
if [ -f /etc/bashrc ]; then
	. /etc/bashrc
fi

# User specific aliases and functions

alias rm='rm -i'
alias cp='cp -i'
alias mv='mv -i'

EOF


	cat <<EOF >etc/fstab
${TARGET}2	/	ext2	defaults	1 1
${TARGET}1	/boot	minix	ro,defaults	0 0
EOF

	if [ "$nparts" -gt 2 ] ; then
		cat <<EOF >>etc/fstab
${TARGET}3	/home	ext2	rw,defaults	0 0
EOF
	fi

	cat <<EOF >>etc/fstab
none		/proc	proc	defaults,noauto	0 0
EOF

	cat <<EOF >>etc/motd

Welcome to FiwixOS $OSVERSION for the i386 architecture.

    - Read the /root/README file to know about all the files installed.
    - Refer to the Installation CD-ROM for the source code and the license of
      every single package installed in this system.

EOF

	cat <<EOF >>etc/os-release
NAME="FiwixOS"
VERSION="$OSVERSION"
ID=fiwixos
VERSION_ID=$OSVERSION
PRETTY_NAME="FiwixOS $OSVERSION"
HOME_URL="https://fiwix.org/"

EOF

	cp /busybox usr/sbin
	ln -s /usr/sbin/busybox usr/sbin/loadkmap
	ln -s /usr/sbin/busybox usr/bin/mount
	ln -s /usr/sbin/busybox usr/bin/umount
	ln -s /usr/sbin/busybox usr/bin/adduser
	ln -s /usr/sbin/busybox usr/bin/deluser
	ln -s /usr/sbin/busybox usr/bin/addgroup
	ln -s /usr/sbin/busybox usr/bin/delgroup
	ln -s /usr/sbin/busybox usr/bin/passwd

	cat <<EOF >>etc/rc.d/rc.local
echo "Setting console keymap."
loadkmap </etc/i18n/$keymap
EOF

	sed -i 's/VERSION/'$OSVERSION'/' etc/issue etc/rc.d/rc.sysinit

	# activate system daemons
	/mnt/disk/usr/bin/chroot /mnt/disk sbin/chkconfig --add atd
	/mnt/disk/usr/bin/chroot /mnt/disk sbin/chkconfig --add crond
	sync
}

build_manpages_db() {
	/mnt/disk/usr/bin/chroot /mnt/disk sbin/mandocdb
	sync
}

set_root_password() {
	local text="
You need to set a password for 'root', the system administrative account.
Note that you will not be able to see the password as you type it.

"
	cd /

	$DIALOG --backtitle "$HEADER" --title "[ SETTING ROOT PASSWORD ]" --infobox "$text" 8 65
	/mnt/disk/usr/bin/chroot /mnt/disk passwd root
	sync
}

install_grub() {
	mount -t minix ${TARGET}1 /mnt/boot || error || exit 1
	mkdir -p /mnt/boot/grub

	cat <<EOF >/mnt/boot/grub/device.map
# this device map was generated during the FiwixOS installation
(hd0)     /dev/hda
EOF

	cat <<EOF >/mnt/boot/grub/menu.lst
# menu.lst generated during the FiwixOS installation
#
# FiwixOS
# Copyright (C) 2018-2022 Jordi Sanfeliu <jordi@fibranet.cat>
#
# GRUB configuration file
#
default 0
timeout 5
title FiwixOS $OSVERSION (standard VGA)
	root (hd0,0)
	kernel /fiwix root=${TARGET}2
title FiwixOS $OSVERSION (VESA FB 1024x768 32bpp)
	root (hd0,0)
	kernel /fiwix root=${TARGET}2
	vbematch 1024 768 32
title FiwixOS $OSVERSION (VESA FB 1024x768 24bpp)
	root (hd0,0)
	kernel /fiwix root=${TARGET}2
	vbematch 1024 768 24
title FiwixOS $OSVERSION (VESA FB 800x600 32bpp)
	root (hd0,0)
	kernel /fiwix root=${TARGET}2
	vbematch 800 600 32
title FiwixOS $OSVERSION (VESA FB 800x600 24bpp)
	root (hd0,0)
	kernel /fiwix root=${TARGET}2
	vbematch 800 600 24
EOF

	ln -s menu.lst /mnt/boot/grub/grub.conf

	sync ; sync
	grub-install --root-directory=/mnt $TARGET || error || exit 1

	# cleaning up
	rm -f /mnt/boot/grub/default
	rm -f /usr/lib 2>/dev/null
	rm -f /usr/sbin/grub* 2>/dev/null
	umount /mnt/boot
}

install_kernel() {
	mount -t minix ${TARGET}1 /mnt/disk/boot || error || exit 1
	system "opkg install -V0 -o /mnt/disk --force-overwrite ${SOURCE}/install/base/kernel_${KVERSION}_i386.ipk" || error || exit 1
	umount /mnt/disk/boot

	if ! grep -q "/dev/root / iso9660" /etc/mtab ; then
		umount /mnt/cdrom
	fi
	rm -rf /mnt/disk/tmp/*
	umount /mnt/disk || error || exit 1
	umount /mnt/ram || error || exit 1
	sync ; sync
}

completed() {
	local text="
The installation of FiwixOS $OSVERSION is complete.

When you press ENTER, the system will be shut down and restarted.

*** MAKE SURE THE BOOT DRIVES (CD-ROM AND/OR DISKETTE) ARE EMPTY ***

"
	$DIALOG --backtitle "$HEADER" --title "[ INSTALLATION TERMINATED ]" --msgbox "$text" 0 0
	trap - EXIT SIGINT SIGTERM
	reboot
}


# Main
# -----------------------------------------------------------------------------

trap cleanup EXIT
trap abort SIGINT SIGTERM

TARGET=
disksize=
SOURCE=
progress=
pass=1

$BRIGHT ; $BGBLUE
scroll ; welcome
info1
info2
info3
check_disk_drive
setup_ramdisk

while : ; do
	case $pass in
		0) press_enter "exit" ;;
		1) keyboard_selection ;;
		2) target_media_selection ;;
		3) last_warning ;;
		*) break ;;
	esac
done

pass=1

while : ; do
	case $pass in
		1) progress="Initializing disk partition block" ;;
		2) progress=$(echo "$progress\nCreating partition 1 for /boot") ;;
		3) progress=$(echo "$progress\nCreating partition 2 for /") ;;
		4) if [ "$nparts" -gt 2 ] ; then
			progress=$(echo "$progress\nCreating partition 3 for /home")
		   fi ;;
		5) progress=$(echo "$progress\nMaking /boot filesystem") ;;
		6) progress=$(echo "$progress\nMaking / filesystem") ;;
		7) if [ "$nparts" -gt 2 ] ; then
			progress=$(echo "$progress\nMaking /home filesystem")
		   fi ;;
		8) progress=$(echo "$progress\nActivating /boot partition") ;;
		9) ;;
		10) progress="Preparing root filesystem" ;;
		11) progress=$(echo "$progress\nCreating devices") ;;
		12) progress=$(echo "$progress\nCreating system directories") ;;
		13) progress=$(echo "$progress\nInstalling system software") ;;
		14) progress=$(echo "$progress\nConfiguring system files") ;;
		15) progress=$(echo "$progress\nBuilding manpages database") ;;
		16) progress=$(echo "$progress\nSetting 'root' password") ;;
		17) progress=$(echo "$progress\nInstalling GRUB") ;;
		18) progress=$(echo "$progress\nInstalling Fiwix kernel") ;;
		*) break ;;
	esac

	$DIALOG --backtitle "$HEADER" --title "[ CONFIGURING $TARGET ]" --infobox "$progress" 10 50

	case $pass in
		1) initializing_disk_partition_block ;;
		2) create_boot_partition ;;
		3) create_root_partition ;;
		4) create_home_partition ;;
		5) make_boot_fs ;;
		6) make_root_fs ;;
		7) make_home_fs ;;
		8) activate_boot ;;
		9) show_target_layout ; start_copy ;;
		10) mount_root_fs ;;
		11) create_devices ;;
		12) create_system_dirs ;;
		13) install_system_software ;;
		14) configure_system_files ;;
		15) build_manpages_db ;;
		16) set_root_password ;;
		17) install_grub ;;
		18) install_kernel ;;
		*) break ;;
	esac

	pass=$(expr $pass + 1)
done

completed
exit 0
